@@ -2,7 +2,7 @@ package App::perlbrew;
 use strict;
 use warnings;
 use 5.008;
-our $VERSION = "0.71";
+our $VERSION = "0.74";
 use Config;
 
 BEGIN {
@@ -19,9 +19,17 @@ BEGIN {
     @INC = @oldinc;
 }
 
-use List::Util qw/min/;
+use File::Glob ':glob';
 use Getopt::Long ();
 
+sub min(@) {
+    my $m = $_[0];
+    for(@_) {
+        $m = $_ if $_ < $m;
+    }
+    return $m;
+}
+
 sub uniq {
     my %seen; grep { !$seen{$_}++ } @_;
 }
@@ -88,6 +96,7 @@ for (@flavors) {
 ### functions
 
 sub joinpath { join "/", @_ }
+sub splitpath { split "/", $_[0] }
 
 sub mkpath {
     require File::Path;
@@ -120,20 +129,20 @@ sub files_are_the_same {
     my %commands = (
         curl => {
             test     => '--version >/dev/null 2>&1',
-            get      => '--insecure --silent --location --fail -o - {url}',
-            download => '--insecure --silent --location --fail -o {output} {url}',
+            get      => '--silent --location --fail -o - {url}',
+            download => '--silent --location --fail -o {output} {url}',
             order    => 1,
         },
         wget => {
             test     => '--version >/dev/null 2>&1',
-            get      => '--no-check-certificate --quiet -O - {url}',
-            download => '--no-check-certificate --quiet -O {output} {url}',
+            get      => '--quiet -O - {url}',
+            download => '--quiet -O {output} {url}',
             order    => 2,
         },
         fetch => {
             test     => '--version >/dev/null 2>&1',
-            get      => '--no-verify-peer -o - {url}',
-            download => '--no-verify-peer {url}',
+            get      => '-o - {url}',
+            download => '{url}',
             order    => 3,
         }
     );
@@ -326,6 +335,7 @@ sub parse_cmdline {
         'root=s',
         'switch',
         'all',
+        'shell=s',
 
         # options passed directly to Configure
         'D=s@',
@@ -369,6 +379,16 @@ sub current_lib {
     return $self->{current_lib} || $self->env('PERLBREW_LIB')  || '';
 }
 
+sub current_shell {
+    my ($self, $x) = @_;
+    $self->{current_shell} = $x if $x;
+    return $self->{current_shell} ||= do {
+        my $shell_name = (splitpath($self->{shell} || $self->env('SHELL')))[-1];
+        $shell_name =~ s/\d+$//;
+        $shell_name;
+    };
+}
+
 sub current_env {
     my ($self) = @_;
     my $l = $self->current_lib;
@@ -424,7 +444,7 @@ sub env {
 sub path_with_tilde {
     my ($self, $dir) = @_;
     my $home = $self->env('HOME');
-    $dir =~ s/^\Q$home\E/~/ if $home;
+    $dir =~ s!\Q$home/\E!~/! if $home;
     return $dir;
 }
 
@@ -536,7 +556,7 @@ sub run_command_version {
 }
 
 sub run_command_help {
-    my ($self, $status, $verbose) = @_;
+    my ($self, $status, $verbose, $return_text) = @_;
 
     require Pod::Usage;
 
@@ -562,6 +582,7 @@ sub run_command_help {
                 $out = "Cannot find documentation for '$status'\n\n";
             }
 
+            return "\n$out" if ($return_text);
             print "\n$out";
             close $fh;
         }
@@ -730,9 +751,7 @@ sub run_command_init {
     my @args = @_;
 
     if (@args && $args[0] eq '-') {
-        if ($self->is_shell_csh) {
-        }
-        else {
+        if ($self->current_shell =~ /(ba|z)?sh/) {
             $self->run_command_init_in_bash;
         }
         exit 0;
@@ -771,16 +790,15 @@ sub run_command_init {
     }
 
     my ( $shrc, $yourshrc );
-    if ( $self->is_shell_csh) {
+    if ( $self->current_shell =~ m/(t?csh)/ ) {
         $shrc     = 'cshrc';
-        $self->env("SHELL") =~ m/(t?csh)/;
         $yourshrc = $1 . "rc";
     }
-    elsif ($self->env("SHELL") =~ m/zsh\d?$/) {
+    elsif ($self->current_shell =~ m/zsh\d?$/) {
         $shrc = "bashrc";
         $yourshrc = 'zshenv';
     }
-    elsif( $self->env('SHELL') =~ m/fish/ ) {
+    elsif( $self->current_shell eq 'fish' ) {
         $shrc = "perlbrew.fish";
         $yourshrc = 'config/fish/config.fish';
     }
@@ -793,7 +811,7 @@ sub run_command_init {
     my $pb_home_dir = $self->path_with_tilde($PERLBREW_HOME);
 
     my $code = qq(    source $root_dir/etc/${shrc});
-    if ($PERLBREW_HOME ne joinpath($ENV{HOME}, ".perlbrew")) {
+    if ($PERLBREW_HOME ne joinpath($self->env('HOME'), ".perlbrew")) {
         $code = "    export PERLBREW_HOME=$pb_home_dir\n" . $code;
     }
 
@@ -1385,6 +1403,9 @@ INSTALL
         unless (-e $newperl) {
             $self->run_command_symlink_executables($installation_name);
         }
+
+        eval { $self->append_log('##### Brew Finished #####') };
+
         if ( $sitecustomize ) {
             my $capture = $self->do_capture("$newperl -V:sitelib");
             my ($sitelib) = $capture =~ /sitelib='(.*)';/;
@@ -1396,9 +1417,19 @@ INSTALL
                 or die "Could not open '$sitecustomize' for reading: $!\n";
             print {$dst} do { local $/; <$src> };
         }
+
+        my $version_file =
+          joinpath( $self->root, 'perls', $installation_name, '.version' );
+
+        if ( -e $version_file ) {
+            unlink($version_file)
+              or die "Could not unlink $version_file file: $!\n";
+        }
+
         print "$installation_name is successfully installed.\n";
     }
     else {
+        eval { $self->append_log('##### Brew Failed #####') };
         die $self->INSTALLATION_FAILURE_MESSAGE;
     }
     return;
@@ -1423,7 +1454,7 @@ sub do_install_program_from_url {
     my $body = http_get($url) or die "\nERROR: Failed to retrieve $program_name executable.\n\n";
 
     unless ($body =~ m{\A#!/}s) {
-        my $x = joinpath($ENV{TMPDIR} || "/tmp", "${program_name}.downloaded.$$");
+        my $x = joinpath($self->env('TMPDIR') || "/tmp", "${program_name}.downloaded.$$");
         my $message = "\nERROR: The downloaded $program_name program seem to be invalid. Please check if the following URL can be reached correctly\n\n\t$url\n\n...and try again latter.";
 
         unless (-f $x) {
@@ -1510,7 +1541,7 @@ sub installed_perls {
             name        => $name,
             orig_version=> $orig_version,
             version     => $self->format_perl_version($orig_version),
-            is_current  => ($self->current_perl eq $name) && !$self->env("PERLBREW_LIB"),
+            is_current  => ($self->current_perl eq $name) && !($self->current_lib),
             libs => [ $self->local_libs($name) ],
             executable  => $executable
         };
@@ -1522,7 +1553,7 @@ sub installed_perls {
 sub local_libs {
     my ($self, $perl_name) = @_;
 
-    my @libs = map { substr($_, length($PERLBREW_HOME) + 6) } <$PERLBREW_HOME/libs/*>;
+    my @libs = map { substr($_, length($PERLBREW_HOME) + 6) } bsd_glob("$PERLBREW_HOME/libs/*");
 
     if ($perl_name) {
         @libs = grep { /^$perl_name\@/ } @libs;
@@ -1566,6 +1597,10 @@ sub perlbrew_env {
         unless ($perl_name) {
             die "\nERROR: The installation \"$name\" is unknown.\n\n";
         }
+
+        unless (!$lib_name || grep { $_->{lib_name} eq $lib_name } $self->local_libs($perl_name)) {
+            die "\nERROR: The lib name \"$lib_name\" is unknown.\n\n";
+        }
     }
 
     my %env = (
@@ -1597,6 +1632,13 @@ sub perlbrew_env {
 
             if (-d $base) {
                 $current_local_lib_context = $current_local_lib_context->activate($base);
+
+                if ( $self->env('PERLBREW_LIB_PREFIX') ) {
+                    unshift
+                        @{$current_local_lib_context->libs},
+                            $self->env('PERLBREW_LIB_PREFIX');
+                }
+
                 $env{PERLBREW_PATH}    = joinpath($base, "bin") . ":" . $env{PERLBREW_PATH};
                 $env{PERLBREW_MANPATH} = joinpath($base, "man") . ":" . $env{PERLBREW_MANPATH};
                 $env{PERLBREW_LIB}  = $lib_name;
@@ -1675,9 +1717,6 @@ WARNINGONMAC
 
         }
     }
-    elsif  ($shell =~ /\/bash$/)  {
-        $shell_opt = "--noprofile --norc";
-    }
 
     my %env = ($self->perlbrew_env($name), PERLBREW_SKIP_INIT => 1);
 
@@ -1707,6 +1746,7 @@ sub run_command_use {
 
     if ( !$perl ) {
         my $current = $self->current_perl;
+        $current .= '@' . $self->current_lib if ($self->current_lib);
         if ($current) {
             print "Currently using $current\n";
         } else {
@@ -1724,6 +1764,7 @@ sub run_command_switch {
 
     unless ( $dist ) {
         my $current = $self->current_perl;
+        $current .= '@' . $self->current_lib if ($self->current_lib);
         printf "Currently switched %s\n",
             ( $current ? "to $current" : 'off' );
         return;
@@ -1771,103 +1812,41 @@ sub run_command_switch_off {
     print "To immediately make it effective, run this line in this terminal:\n\n    exec @{[ $self->env('SHELL') ]}\n\n";
 }
 
-sub run_command_mirror {
-    my($self) = @_;
-    print "Fetching mirror list\n";
-    my $raw = http_get("http://search.cpan.org/mirror");
-
-    unless ($raw) {
-        die "\nERROR: Failed to retrieve the mirror list.\n\n";
-    }
-
-    my $found;
-    my @mirrors;
-    foreach my $line ( split m{\n}, $raw ) {
-        $found = 1 if $line =~ m{<select name="mirror">};
-        next if ! $found;
-        last if $line =~ m{</select>};
-        if ( $line =~ m{<option value="(.+?)">(.+?)</option>} ) {
-            my $url  = $1;
-            my $name = $2;
-            $name =~ s/&#(\d+);/chr $1/seg;
-            $url =~ s/&#(\d+);/chr $1/seg;
-            push @mirrors, { url => $url, name => $name };
-        }
-    }
-
-    require ExtUtils::MakeMaker;
-    my $select;
-    my $max = @mirrors;
-    my $id  = 0;
-    while ( @mirrors ) {
-        my @page = splice(@mirrors,0,20);
-        my $base = $id;
-        printf "[% 3d] %s\n", ++$id, $_->{name} for @page;
-        my $remaining = $max - $id;
-        my $ask = "Select a mirror by number or press enter to see the rest "
-                . "($remaining more) [q to quit, m for manual entry]";
-        my $val = ExtUtils::MakeMaker::prompt( $ask );
-        if ( ! length $val )  { next }
-        elsif ( $val eq 'q' ) { last }
-        elsif ( $val eq 'm' ) {
-            my $url  = ExtUtils::MakeMaker::prompt("Enter the URL of your CPAN mirror:");
-            my $name = ExtUtils::MakeMaker::prompt("Enter a Name: [default: My CPAN Mirror]") || "My CPAN Mirror";
-            $select = { name => $name, url => $url };
-            last;
-        }
-        elsif ( not $val =~ /\s*(\d+)\s*/ ) {
-            die "Invalid answer: must be 'q', 'm' or a number\n";
-        }
-        elsif (1 <= $val and $val <= $max) {
-            $select = $page[ $val - 1 - $base ];
-            last;
-        }
-        else {
-            die "Invalid ID: must be between 1 and $max\n";
-        }
-    }
-    die "You didn't select a mirror!\n" if ! $select;
-    print "Selected $select->{name} ($select->{url}) as the mirror\n";
-    my $conf = $self->config;
-    $conf->{mirror} = $select;
-    $self->_save_config;
-    return;
-}
-
 sub run_command_env {
     my($self, $name) = @_;
 
     my %env = $self->perlbrew_env($name);
 
-    if ($self->env('SHELL') =~ /(ba|k|z|\/)sh\d?$/) {
-        for my $k (sort keys %env) {
-            my $v = $env{$k};
-            if (defined $v) {
-                if ($v || ($v && exists($ENV{$k}))) {
-                    $v =~ s/(\\")/\\$1/g;
-                    print "export $k=\"$v\"\n";
-                }
-            }
-            else {
-                if (exists $ENV{$k}) {
-                    print "unset $k\n";
-                }
+    my @statements;
+    for my $k (sort keys %env) {
+        my $v = $env{$k};
+        if (defined($v) && $v ne '') {
+            $v =~ s/(\\")/\\$1/g;
+            push @statements, ["set", $k, $v];
+        } else {
+            if (exists $ENV{$k}) {
+                push @statements, ["unset", $k];
             }
         }
     }
-    else {
-        for my $k (sort keys %env) {
-            my $v = $env{$k};
-            if (defined $v) {
-                if ($v || ($v && exists($ENV{$k}))) {
-                    $v =~ s/(\\")/\\$1/g;
-                    print "setenv $k \"$v\"\n";
-                }
+
+    if ($self->env('SHELL') =~ /(ba|k|z|\/)sh\d?$/) {
+        for (@statements) {
+            my ($o,$k,$v) = @$_;
+            if ($o eq 'unset') {
+                print "unset $k\n";
+            } else {
+                $v =~ s/(\\")/\\$1/g;
+                print "export $k=\"$v\"\n";
             }
-            else {
-                if (exists $ENV{$k}) {
-                    print "unsetenv $k\n";
-                }
+        }
+    } else {
+        for (@statements) {
+            my ($o,$k,$v) = @$_;
+            if ($o eq 'unset') {
+                print "unsetenv $k\n";
+            } else {
+                print "setenv $k \"$v\"\n";
             }
         }
     }
@@ -1978,7 +1957,7 @@ sub run_command_exec {
     if ($opts{with}) {
         my %installed = map { $_->{name} => $_ } map { ($_, @{$_->{libs}}) } $self->installed_perls;
 
-        my $d = ($opts{with} =~ / /) ? qr( +) : qr(,+);
+        my $d = ($opts{with} =~ m/ /) ? qr( +) : qr(,+);
         my @with = grep { $_ } map {
             my ($p,$l) = $self->resolve_installation_name($_);
             $p .= "\@$l" if $l;
@@ -2014,12 +1993,15 @@ sub run_command_exec {
             # return 255 for case when process was terminated with signal, in that case real exit code is useless and weird
             $exit_code = 255 if $exit_code > 255;
             $overall_success = 0;
-            print "Command terminated with non-zero status.\n" unless $self->{quiet};
 
-            print STDERR "Command [" .
-                join(' ', map { /\s/ ? "'$_'" : $_ } @ARGV) . # trying reverse shell escapes - quote arguments containing spaces
-                "] terminated with exit code $exit_code (\$? = $err) under the following perl environment:\n";
-            print STDERR $self->format_info_output;
+            unless ($self->{quiet}) {
+                print "Command terminated with non-zero status.\n";
+
+                print STDERR "Command [" .
+                    join(' ', map { /\s/ ? "'$_'" : $_ } @ARGV) . # trying reverse shell escapes - quote arguments containing spaces
+                    "] terminated with exit code $exit_code (\$? = $err) under the following perl environment:\n";
+                print STDERR $self->format_info_output;
+            }
 
             $self->do_exit_with_error_code($exit_code) if ($opts{'halt-on-error'});
         }
@@ -2050,18 +2032,9 @@ sub run_command_clean {
 sub run_command_alias {
     my ($self, $cmd, $name, $alias) = @_;
 
-    if (!$cmd) {
-        print <<USAGE;
-
-Usage: perlbrew alias [-f] <action> <name> [<alias>]
-
-    perlbrew alias create <name> <alias>
-    perlbrew alias delete <alias>
-    perlbrew alias rename <old_alias> <new_alias>
-
-USAGE
-
-        return;
+    unless($cmd) {
+        $self->run_command_help("alias");
+        exit(-1);
     }
 
     my $path_name  = joinpath($self->root, "perls", $name);
@@ -2124,29 +2097,12 @@ sub run_command_display_installation_failure_message {
     my ($self) = @_;
 }
 
-sub lib_usage {
-    my $usage = <<'USAGE';
-
-Usage: perlbrew lib <action> <name> [<name> <name> ...]
-
-    perlbrew lib list
-    perlbrew lib create nobita
-    perlbrew lib create perl-5.14.2@nobita
-
-    perlbrew use perl-5.14.2@nobita
-    perlbrew lib delete perl-5.12.3@nobita shizuka
-
-USAGE
-
-
-    return $usage;
-}
-
 sub run_command_lib {
     my ($self, $subcommand, @args) = @_;
+
     unless ($subcommand) {
-        print lib_usage;
-        return;
+        $self->run_command_help("lib");
+        exit(-1);
     }
 
     my $sub = "run_command_lib_$subcommand";
@@ -2161,7 +2117,7 @@ sub run_command_lib {
 sub run_command_lib_create {
     my ($self, $name) = @_;
 
-    die "ERROR: No lib name\n", lib_usage unless $name;
+    die "ERROR: No lib name\n", $self->run_command_help("lib", undef, 'return_text') unless $name;
 
     $name =~ s/^/@/ unless $name =~ /@/;
 
@@ -2190,7 +2146,7 @@ sub run_command_lib_create {
 sub run_command_lib_delete {
     my ($self, $name) = @_;
 
-    die "ERROR: No lib to delete\n", lib_usage unless $name;
+    die "ERROR: No lib to delete\n", $self->run_command_help("lib", undef, 'return_text') unless $name;
 
     $name =~ s/^/@/ unless $name =~ /@/;
 
@@ -2198,7 +2154,7 @@ sub run_command_lib_delete {
 
     my $fullname = $perl_name . '@' . $lib_name;
 
-    my $current  = $self->current_perl . '@' . ($self->env("PERLBREW_LIB") || "");
+    my $current  = $self->current_perl . '@' . $self->current_lib;
 
     my $dir = joinpath($PERLBREW_HOME,  "libs", $fullname);
 
@@ -2222,18 +2178,13 @@ sub run_command_lib_delete {
 
 sub run_command_lib_list {
     my ($self) = @_;
-
-    my $current = "";
-    if ($self->current_perl && $self->env("PERLBREW_LIB")) {
-        $current = $self->current_perl . "@" . $self->env("PERLBREW_LIB");
-    }
-
     my $dir = joinpath($PERLBREW_HOME,  "libs");
     return unless -d $dir;
 
     opendir my $dh, $dir or die "open $dir failed: $!";
     my @libs = grep { !/^\./ && /\@/ } readdir($dh);
 
+    my $current = $self->current_env;
     for (@libs) {
         print $current eq $_ ? "* " : "  ";
         print "$_\n";
@@ -2287,6 +2238,16 @@ sub run_command_upgrade_perl {
     print "Upgrading $current->{name} to $dist_version\n" unless $self->{quiet};
     local $self->{as}        = $current->{name};
     local $self->{dist_name} = $dist;
+
+    require Config ;
+    my @d_options = map { '-D' . $flavor{$_}->{d_option}} keys %flavor ;
+    my %sub_config = map { $_ => $Config{$_}} grep { /^config_arg\d/} keys %Config ;
+    for my $value ( values %sub_config ) {
+        my $value_wo_D = $value;
+        $value_wo_D =~ s/^-D//;
+        push @{$self->{D}} , $value_wo_D if grep {/$value/} @d_options;
+    }
+    
     $self->do_install_release($dist, $dist_version);
 }
 
@@ -2406,7 +2367,8 @@ sub _load_config {
 }
 
 sub BASHRC_CONTENT() {
-    return "export PERLBREW_BASHRC_VERSION=$VERSION\n\n" . sprintf <<'RC', $PERLBREW_ROOT;
+    return "export PERLBREW_BASHRC_VERSION=$VERSION\n" .
+           (exists $ENV{PERLBREW_ROOT} ? "export PERLBREW_ROOT=$PERLBREW_ROOT\n" : "") . "\n" . <<'RC';
 
 __perlbrew_reinit() {
     if [[ ! -d "$PERLBREW_HOME" ]]; then
@@ -2419,45 +2381,35 @@ __perlbrew_reinit() {
     __perlbrew_set_path
 }
 
+__perlbrew_purify () {
+    local path patharray outsep
+    IFS=: read -r${BASH_VERSION+a}${ZSH_VERSION+A} patharray <<< "$1"
+    for path in "${patharray[@]}" ; do
+        case "$path" in
+            (*"$PERLBREW_HOME"*) ;;
+            (*"$PERLBREW_ROOT"*) ;;
+            (*) printf '%s' "$outsep$path" ; outsep=: ;;
+        esac
+    done
+}
+
 __perlbrew_set_path () {
-    MANPATH_WITHOUT_PERLBREW=`perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_HOME}) < 0 } grep { index($_, $ENV{PERLBREW_ROOT}) < 0 } split/:/,qx(manpath 2> /dev/null);'`
-    if [ -n "$PERLBREW_MANPATH" ]; then
-        export MANPATH="$PERLBREW_MANPATH:$MANPATH_WITHOUT_PERLBREW"
-    else
-        export MANPATH="$MANPATH_WITHOUT_PERLBREW"
-    fi
-    unset MANPATH_WITHOUT_PERLBREW
-
-    PATH_WITHOUT_PERLBREW=$(eval $perlbrew_command display-pristine-path)
-    if [ -n "$PERLBREW_PATH" ]; then
-        export PATH=${PERLBREW_PATH}:${PATH_WITHOUT_PERLBREW}
-    else
-        export PATH=${PERLBREW_ROOT}/bin:${PATH_WITHOUT_PERLBREW}
-    fi
-    unset PATH_WITHOUT_PERLBREW
-
+    export MANPATH=$PERLBREW_MANPATH${PERLBREW_MANPATH:+:}$(__perlbrew_purify "$(manpath 2>/dev/null)")
+    export PATH=${PERLBREW_PATH:-$PERLBREW_ROOT/bin}:$(__perlbrew_purify "$PATH")
     hash -r
 }
 
 __perlbrew_set_env() {
-    local code="$($perlbrew_command env $@)"
-    local exit_status="$?"
-    if [[ $exit_status -eq 0 ]] ; then
-        eval "$code"
-    else
-        return $exit_status
-    fi
+    local code
+    code="$($perlbrew_command env $@)" || return $?
+    eval "$code"
 }
 
 __perlbrew_activate() {
     [[ -n $(alias perl 2>/dev/null) ]] && unalias perl 2>/dev/null
 
     if [[ -n "$PERLBREW_PERL" ]]; then
-        if [[ -z "$PERLBREW_LIB" ]]; then
-            __perlbrew_set_env $PERLBREW_PERL
-        else
-            __perlbrew_set_env $PERLBREW_PERL@$PERLBREW_LIB
-        fi
+        __perlbrew_set_env "$PERLBREW_PERL${PERLBREW_LIB:+@}$PERLBREW_LIB"
     fi
 
     __perlbrew_set_path
@@ -2485,18 +2437,12 @@ perlbrew () {
     case $1 in
         (use)
             if [[ -z "$2" ]] ; then
-                if [[ -z "$PERLBREW_PERL" ]] ; then
-                    echo "Currently using system perl"
-                else
-                    echo "Currently using $PERLBREW_PERL"
-                fi
+                echo -n "Currently using ${PERLBREW_PERL:-system perl}"
+                [ -n "$PERLBREW_LIB" ] && echo -n "@$PERLBREW_LIB"
+                echo
             else
-                __perlbrew_set_env "$2"
+                __perlbrew_set_env "$2" && { __perlbrew_set_path ; true ; }
                 exit_status="$?"
-                if [[ $exit_status -eq 0 ]]
-                then
-                    __perlbrew_set_path
-                fi
             fi
             ;;
 
@@ -2504,11 +2450,8 @@ perlbrew () {
               if [[ -z "$2" ]] ; then
                   command perlbrew switch
               else
-                  perlbrew use $2
+                  perlbrew use $2 && { __perlbrew_reinit $2 ; true ; }
                   exit_status=$?
-                  if [[ ${exit_status} -eq 0 ]]; then
-                      __perlbrew_reinit $2
-                  fi
               fi
               ;;
 
@@ -2532,7 +2475,7 @@ perlbrew () {
     return ${exit_status:-0}
 }
 
-[[ -z "$PERLBREW_ROOT" ]] && export PERLBREW_ROOT="%s"
+[[ -z "$PERLBREW_ROOT" ]] && export PERLBREW_ROOT="$HOME/perl5/perlbrew"
 [[ -z "$PERLBREW_HOME" ]] && export PERLBREW_HOME="$HOME/.perlbrew"
 
 if [[ ! -n "$PERLBREW_SKIP_INIT" ]]; then
@@ -2585,26 +2528,28 @@ function __perlbrew_reinit
 end
 
 function __perlbrew_set_path
-    set -l MANPATH_WITHOUT_PERLBREW (perl -e 'print join " ", grep { index($_, $ENV{PERLBREW_HOME}) < 0 } grep { index($_, $ENV{PERLBREW_ROOT}) < 0 } split/:/,qx(manpath 2> /dev/null);')
+    set -l MANPATH_WITHOUT_PERLBREW (perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_HOME}) < 0 } grep { index($_, $ENV{PERLBREW_ROOT}) < 0 } split/:/,qx(manpath 2> /dev/null);')
 
     if test -n "$PERLBREW_MANPATH"
-        set -x MANPATH $PERLBREW_MANPATH $MANPATH_WITHOUT_PERLBREW
+        set -l PERLBREW_MANPATH $PERLBREW_MANPATH":"
+        set -x MANPATH {$PERLBREW_MANPATH}{$MANPATH_WITHOUT_PERLBREW}
     else
         set -x MANPATH $MANPATH_WITHOUT_PERLBREW
     end
 
     set -l PATH_WITHOUT_PERLBREW (eval $perlbrew_command display-pristine-path | perl -pe'y/:/ /')
 
+    # silencing stderr in case there's a non-existent path in $PATH (see GH#446)
     if test -n "$PERLBREW_PATH"
         set -x PERLBREW_PATH (echo $PERLBREW_PATH | perl -pe 'y/:/ /' )
-        eval set -x PATH $PERLBREW_PATH $PATH_WITHOUT_PERLBREW
+        eval set -x PATH $PERLBREW_PATH $PATH_WITHOUT_PERLBREW 2> /dev/null
     else
-        eval set -x PATH $PERLBREW_ROOT/bin $PATH_WITHOUT_PERLBREW
+        eval set -x PATH $PERLBREW_ROOT/bin $PATH_WITHOUT_PERLBREW 2> /dev/null
     end
 end
 
 function __perlbrew_set_env
-    set -l code (eval $perlbrew_command env $argv | perl -pe 's/export\s+(\S+)="(\S*)"/set -x $1 $2;/g; y/:/ /')
+    set -l code (eval $perlbrew_command env $argv | perl -pe 's/^(export|setenv)/set -xg/; s/=/ /; s/^unset[env]*/set -eug/; s/$/;/; y/:/ /')
 
     if test -z "$code"
         return 0;
@@ -2682,7 +2627,7 @@ function perlbrew
 end
 
 function __source_init
-    perl -pe's/^export/set -x/; s/=/ /; s/$/;/;' "$PERLBREW_HOME/init" | . -
+    perl -pe's/^(export|setenv)/set -xg/; s/=/ /; s/$/;/;' "$PERLBREW_HOME/init" | source
 end
 
 if test -z "$PERLBREW_ROOT"
@@ -2870,6 +2815,15 @@ CSHRC
 
 }
 
+sub append_log {
+    my ($self, $message) = @_;
+    my $log_handler;
+    open($log_handler, '>>', $self->{log_file})
+        or die "Cannot open log file for appending: $!";
+    print $log_handler "$message\n";
+    close($log_handler);
+}
+
 sub INSTALLATION_FAILURE_MESSAGE {
     my ($self) = @_;
     return <<FAIL;
@@ -2877,7 +2831,7 @@ Installation process failed. To spot any issues, check
 
   $self->{log_file}
 
-If some perl tests failed and you still want install this distribution anyway,
+If some perl tests failed and you still want to install this distribution anyway,
 do:
 
   (cd $self->{dist_extracted_dir}; make install)
@@ -2904,12 +2858,12 @@ __END__
 
 =head1 NAME
 
-App::perlbrew - Manage perl installations in your $HOME
+L<App::perlbrew> - Manage perl installations in your C<$HOME>
 
-=head1 SYNOPSIS
+=head2 SYNOPSIS
 
     # Installation
-    curl -kL http://install.perlbrew.pl | bash
+    curl -L http://install.perlbrew.pl | bash
 
     # Initialize
     perlbrew init
@@ -2945,28 +2899,29 @@ App::perlbrew - Manage perl installations in your $HOME
     # Exec something with all perlbrew-ed perls
     perlbrew exec -- perl -E 'say $]'
 
-=head1 DESCRIPTION
+=head2 DESCRIPTION
 
-perlbrew is a program to automate the building and installation of perl in an
+L<perlbrew> is a program to automate the building and installation of perl in an
 easy way. It provides multiple isolated perl environments, and a mechanism
 for you to switch between them.
 
 Everything are installed unter C<~/perl5/perlbrew>. You then need to include a
 bashrc/cshrc provided by perlbrew to tweak the PATH for you. You then can
-benefit from not having to run 'sudo' commands to install
-cpan modules because those are installed inside your HOME too.
+benefit from not having to run C<sudo> commands to install
+cpan modules because those are installed inside your C<HOME> too.
 
 For the documentation of perlbrew usage see L<perlbrew> command
-on CPAN, or by running C<perlbrew help>. The following documentation
+on L<MetaCPAN|https://metacpan.org/>, or by running C<perlbrew help>, 
+or by visiting L<perlbrew's official website|http://perlbrew.pl/>. The following documentation
 features the API of C<App::perlbrew> module, and may not be remotely
 close to what your want to read.
 
-=head1 INSTALLATION
+=head2 INSTALLATION
 
 It is the simplest to use the perlbrew installer, just paste this statement to
 your terminal:
 
-    curl -kL http://install.perlbrew.pl | bash
+    curl -L http://install.perlbrew.pl | bash
 
 Or this one, if you have C<fetch> (default on FreeBSD):
 
@@ -2980,17 +2935,17 @@ The installed perlbrew command is a standalone executable that can be run with
 system perl. The minimum system perl version requirement is 5.8.0, which should
 be good enough for most of the OSes these days.
 
-A fat-packed version of C<patchperl> is also installed to
+A fat-packed version of L<patchperl> is also installed to
 C<~/perl5/perlbrew/bin>, which is required to build old perls.
 
 The directory C<~/perl5/perlbrew> will contain all install perl executables,
 libraries, documentations, lib, site_libs. In the documentation, that directory
-is referred as "perlbrew root". If you need to set it to somewhere else because,
-say, your HOME has limited quota, you can do that by setting C<PERLBREW_ROOT>
+is referred as C<perlbrew root>. If you need to set it to somewhere else because,
+say, your C<HOME> has limited quota, you can do that by setting C<PERLBREW_ROOT>
 environment variable before running the installer:
 
     export PERLBREW_ROOT=/opt/perl5
-    curl -kL http://install.perlbrew.pl | bash
+    curl -L http://install.perlbrew.pl | bash
 
 As a result, different users on the same machine can all share the same perlbrew
 root directory (although only original user that made the installation would
@@ -3016,27 +2971,27 @@ C<App::perlbrew> because it will be installed under a system PATH like
 C</usr/bin>, which is not affected by perlbrew C<switch> or C<use> command.
 
 The C<self-upgrade> command will not upgrade the perlbrew installed by cpan
-command, but it is also easy to upgrade perlbrew by running `cpan App::perlbrew`
+command, but it is also easy to upgrade perlbrew by running C<cpan App::perlbrew>
 again.
 
-=head1 METHODS
+=head2 METHODS
 
 =over 4
 
 =item (Str) current_perl
 
 Return the "current perl" object attribute string, or, if absent, the value of
-PERLBREW_PERL environment variable.
+C<PERLBREW_PERL> environment variable.
 
 =item (Str) current_perl (Str)
 
-Set the "current_perl" object attribute to the given value.
+Set the C<current_perl> object attribute to the given value.
 
 =back
 
-=head1 PROJECT DEVELOPMENT
+=head2 PROJECT DEVELOPMENT
 
-perlbrew project uses github
+L<perlbrew project|http://perlbrew.pl/> uses github
 L<http://github.com/gugod/App-perlbrew/issues> and RT
 <https://rt.cpan.org/Dist/Display.html?Queue=App-perlbrew> for issue
 tracking. Issues sent to these two systems will eventually be reviewed
@@ -3051,13 +3006,13 @@ Kang-min Liu  C<< <gugod@gugod.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2010,2011,2012,2013,2014 Kang-min Liu C<< <gugod@gugod.org> >>.
+Copyright (c) 2010,2011,2012,2013,2014,2015 Kang-min Liu C<< <gugod@gugod.org> >>.
 
-=head1 LICENCE
+=head3 LICENCE
 
 The MIT License
 
-=head1 DISCLAIMER OF WARRANTY
+=head2 DISCLAIMER OF WARRANTY
 
 BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
 FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
